/* antimicro Gamepad to KB+M event mapper
 * Copyright (C) 2015 Travis Nickles <nickles.travis@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef COMMON_H
#define COMMON_H

#include <QtGlobal>
#include <QString>
#include <QDir>
#include <QStringList>
#include <QFileInfo>
#include <QTranslator>
#include <QWaitCondition>
#include <QMutex>
#include <QReadWriteLock>
#include <QThread>

#include "config.h"
#include "antimicrosettings.h"
#include "mousehelper.h"

#ifdef Q_OS_WIN

static QString findWinSystemConfigPath()
{
    QString temp;
    temp = (!qgetenv("LocalAppData").isEmpty()) ?
                QString(qgetenv("LocalAppData")) + "/antimicro" :
                QDir::homePath() + "/.antimicro";
    return temp;
}

static QString findWinLocalConfigPath()
{
    QString temp = QDir::currentPath();
    return temp;
}

static QString findWinDefaultConfigPath()
{
    QString temp = findWinLocalConfigPath();
    QFileInfo dirInfo(temp);
    if (!dirInfo.isWritable())
    {
        temp = findWinSystemConfigPath();
    }

    return temp;
}

static QString findWinConfigPath(QString configFileName)
{
    QString temp;
    QFileInfo localConfigInfo(findWinLocalConfigPath().append("/").append(configFileName));
    QFileInfo systemConfigInfo(findWinSystemConfigPath().append("/").append(configFileName));
    if (localConfigInfo.exists() && localConfigInfo.isWritable())
    {
        temp = localConfigInfo.absoluteFilePath();
    }
    else if (systemConfigInfo.exists() && systemConfigInfo.isWritable())
    {
        temp = systemConfigInfo.absoluteFilePath();
    }
    else
    {
        temp = findWinDefaultConfigPath().append("/").append(configFileName);
    }

    return temp;
}

#endif


namespace PadderCommon
{
#if defined(Q_OS_WIN) && defined(WIN_PORTABLE_PACKAGE)
    const QString configPath = findWinLocalConfigPath();
#elif defined(Q_OS_WIN)
    const QString configPath = findWinSystemConfigPath();
#else
    const QString configPath = (!qgetenv("XDG_CONFIG_HOME").isEmpty()) ?
                QString(qgetenv("XDG_CONFIG_HOME")) + "/antimicro" :
                QDir::homePath() + "/.config/antimicro";

#endif

    const QString configFileName = "antimicro_settings.ini";
#if defined(Q_OS_WIN) && defined(WIN_PORTABLE_PACKAGE)
    const QString configFilePath = QString(configPath).append("/").append(configFileName);
#elif defined(Q_OS_WIN)
    const QString configFilePath = QString(configPath).append("/").append(configFileName);
#else
    const QString configFilePath = QString(configPath).append("/").append(configFileName);
#endif

    const int LATESTCONFIGFILEVERSION = 19;
    // Specify the last known profile version that requires a migration
    // to be performed in order to be compatible with the latest version.
    const int LATESTCONFIGMIGRATIONVERSION = 5;
    const QString localSocketKey = "antimicroSignalListener";
    const QString githubProjectPage = "https://github.com/Ryochan7/antimicro";
    const QString wikiPage = QString("%1/wiki").arg(githubProjectPage);
    const QString mouseDeviceName("antimicro Mouse Emulation");
    const QString keyboardDeviceName("antimicro Keyboard Emulation");
    const int ANTIMICRO_MAJOR_VERSION = PROJECT_MAJOR_VERSION;
    const int ANTIMICRO_MINOR_VERSION = PROJECT_MINOR_VERSION;
    const int ANTIMICRO_PATCH_VERSION = PROJECT_PATCH_VERSION;

    const QString programVersion = (ANTIMICRO_PATCH_VERSION > 0) ?
        QString("%1.%2.%3").arg(ANTIMICRO_MAJOR_VERSION)
            .arg(ANTIMICRO_MINOR_VERSION).arg(ANTIMICRO_PATCH_VERSION) :
        QString("%1.%2").arg(ANTIMICRO_MAJOR_VERSION)
            .arg(ANTIMICRO_MINOR_VERSION);

    extern QWaitCondition waitThisOut;
    extern QMutex sdlWaitMutex;
    extern QMutex inputDaemonMutex;
    extern bool editingBindings;
    extern QReadWriteLock editingLock;
    extern MouseHelper mouseHelperObj;

    QString preferredProfileDir(AntiMicroSettings *settings);
    QStringList arguments(int &argc, char **argv);
    void reloadTranslations(QTranslator *translator,
                            QTranslator *appTranslator,
                            QString language);
    void lockInputDevices();
    void unlockInputDevices();
}

Q_DECLARE_METATYPE(QThread*)

#endif // COMMON_H
